from sqlalchemy import (
    Column,
    Integer,
    DateTime,
)
from datetime import datetime

from ...database import Base


class NewswireCompanyUpdateOutboxModel(Base):
    __tablename__ = 'newswire_company_update_outbox'

    id = Column(Integer, primary_key=True)
    news_id = Column(
        Integer,
        nullable=False,
        index=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CollectionEntityResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    collection_id = fields.Integer(required=True)
    entity_type = fields.String(required=True)
    entity_id = fields.String(required=True)
    entity_name = fields.String(required=True)
    is_deleted = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    Text,
)

from ...database import Base


class MergerActionModel(Base):
    __tablename__ = "merger_actions"

    id = Column(Integer, primary_key=True)
    acquirer_company = Column(Text, nullable=False)
    target_company = Column(Text, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
)


class UserCollectionResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    user_id = fields.String(required=True)
    collection_id = fields.Integer(required=True)
    is_deleted = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class PartnershipOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    announcement_date = fields.DateTime(required=True)
    news_id = fields.Integer(required=True)
    partner_one_sec_id = fields.Integer(allow_none=True)
    partner_one_ous_id = fields.Integer(allow_none=True)
    partner_two_sec_id = fields.Integer(allow_none=True)
    partner_two_ous_id = fields.Integer(allow_none=True)
    deal_value = fields.Decimal(allow_none=True)
    type = fields.String(allow_none=True)
    note = fields.String(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['announcement_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format="%Y%m%dT%H%M%S",
        )
        return in_data



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
    UniqueConstraint,
    Float,
    Boolean,
)

from ...database import Base


class InterventionDataInterventionModel(Base):
    __tablename__ = "intervention_data_intervention"

    id = Column(Integer, primary_key=True)
    intervention_data_id = Column(
        Integer,
        ForeignKey('intervention_data.id'),
        nullable=False,
    )
    intervention_id = Column(
        Integer,
        ForeignKey('interventions.id'),
        nullable=False,
    )
    score = Column(
        Float,
        nullable=False,
    )
    preferred = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint('intervention_data_id', 'intervention_id'),)
